require 'specinfra/helper/os'
require 'specinfra/helper/detect_os'

require 'specinfra/helper/backend'
include SpecInfra::Helper::Backend

require 'specinfra/helper/configuration'

require 'specinfra/helper/properties'
include SpecInfra::Helper::Properties
