require "specinfra/command/base"

# Linux
require "specinfra/command/linux"
require "specinfra/command/debian"
require "specinfra/command/ubuntu"
require "specinfra/command/gentoo"
require "specinfra/command/plamo"
require "specinfra/command/redhat"

# Solaris
require "specinfra/command/solaris"
require "specinfra/command/solaris10"
require "specinfra/command/solaris11"
require "specinfra/command/smartos"

# Others
require "specinfra/command/aix"
require "specinfra/command/darwin"
require "specinfra/command/freebsd"
require "specinfra/command/windows"

